/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "user_callbacks.h"
#include "main.h"

#include <system_settings.h>

#define SYS_INFO_COUNT 6

typedef enum {
    BOOL,
    INT,
    STRING
} system_settings_return_types_e;

struct _ret_type_define {
    system_settings_key_e key;
    system_settings_return_types_e returns;
};

struct _ret_type_define _ret_type[] = {
    {
        SYSTEM_SETTINGS_KEY_INCOMING_CALL_RINGTONE, STRING
    },
    {
        SYSTEM_SETTINGS_KEY_WALLPAPER_HOME_SCREEN, STRING
    },
    {
        SYSTEM_SETTINGS_KEY_WALLPAPER_LOCK_SCREEN, STRING
    },
    {
        SYSTEM_SETTINGS_KEY_FONT_SIZE, INT
    },
    {
        SYSTEM_SETTINGS_KEY_FONT_TYPE, STRING
    },
    {
        SYSTEM_SETTINGS_KEY_MOTION_ACTIVATION, BOOL
    }
};

// Function used for mapping system settings key to its string representation.
static const char *_map_system_settings_key(system_settings_key_e key)
{
    switch (key) {
    case SYSTEM_SETTINGS_KEY_INCOMING_CALL_RINGTONE:
        return "SYSTEM_SETTINGS_KEY_INCOMING_CALL_RINGTONE";

    case SYSTEM_SETTINGS_KEY_WALLPAPER_HOME_SCREEN:
        return "SYSTEM_SETTINGS_KEY_WALLPAPER_HOME_SCREEN";

    case SYSTEM_SETTINGS_KEY_WALLPAPER_LOCK_SCREEN:
        return "SYSTEM_SETTINGS_KEY_WALLPAPER_LOCK_SCREEN";

    case SYSTEM_SETTINGS_KEY_FONT_SIZE:
        return "SYSTEM_SETTINGS_KEY_FONT_SIZE";

    case SYSTEM_SETTINGS_KEY_FONT_TYPE:
        return "SYSTEM_SETTINGS_KEY_FONT_TYPE";

    case SYSTEM_SETTINGS_KEY_MOTION_ACTIVATION:
        return "SYSTEM_SETTINGS_KEY_MOTION_ACTIVATION";

    default:
        return "OTHER";
    }
}

static void _system_settings_changed_cb(system_settings_key_e key, void *user_data)
{
    dlog_print(DLOG_ERROR, LOG_TAG, "Runtime Info changed: %s", _map_system_settings_key(key));
    PRINT_MSG("Runtime Info changed: %s", _map_system_settings_key(key));
}

static void _system_settings_cb(void *data, Evas_Object *obj, void *event_info)
{
    int ret;

    int i;

    for (i = 0; i < SYS_INFO_COUNT; i++) {
        PRINT_MSG("%d. %s : ", i + 1, _map_system_settings_key(_ret_type[i].key));
        dlog_print(DLOG_DEBUG, LOG_TAG, "%d. System_settings: %s : ", i + 1,
                   _map_system_settings_key(_ret_type[i].key));
        if (BOOL == _ret_type[i].returns) {
            bool _bool_ret = false;
            ret = system_settings_get_value_bool(_ret_type[i].key, &_bool_ret);
            if (ret != SYSTEM_SETTINGS_ERROR_NONE) {
                PRINT_MSG("system_settings_get_value_bool error: %d", ret);
                dlog_print(DLOG_ERROR, LOG_TAG, "system_settings_get_value_bool error: %d", ret);
            } else {
                PRINT_MSG("%d", _bool_ret);
                dlog_print(DLOG_DEBUG, LOG_TAG, "%d", _bool_ret);
            }
        } else if (INT == _ret_type[i].returns) {
            int _int_ret = 0;
            ret = system_settings_get_value_int(_ret_type[i].key, &_int_ret);
            if (ret != SYSTEM_SETTINGS_ERROR_NONE) {
                PRINT_MSG("system_settings_get_value_int error: %d", ret);
                dlog_print(DLOG_ERROR, LOG_TAG, "system_settings_get_value_int error: %d", ret);
            } else {
                PRINT_MSG("%d", _int_ret);
                dlog_print(DLOG_DEBUG, LOG_TAG, "%d", _int_ret);
            }
        } else if (STRING == _ret_type[i].returns) {
            char *_string_ret = NULL;
            ret = system_settings_get_value_string(_ret_type[i].key, &_string_ret);
            if (ret != SYSTEM_SETTINGS_ERROR_NONE) {
                PRINT_MSG("system_settings_get_value_string error: %d", ret);
                dlog_print(DLOG_ERROR, LOG_TAG, "system_settings_get_value_string error: %d", ret);
            } else {
                PRINT_MSG("%s", _string_ret);
                dlog_print(DLOG_DEBUG, LOG_TAG, "%s", _string_ret);
            }

            free(_string_ret);
        } else {
            PRINT_MSG("Undefined return type");
            dlog_print(DLOG_DEBUG, LOG_TAG, "Undefined return type");
        }
    }
}

static void _set_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
    appdata_s *ad = (appdata_s *)data;

    int i;

    for (i = 0; i < SYS_INFO_COUNT; i++) {
        int ret = system_settings_set_changed_cb(_ret_type[i].key, _system_settings_changed_cb, 0);
        if (ret != SYSTEM_SETTINGS_ERROR_NONE) {
            PRINT_MSG("system_settings_set_changed_cb error: %d", ret);
            dlog_print(DLOG_ERROR, LOG_TAG, "system_settings_set_changed_cb error: %d", ret);
        } else
            PRINT_MSG("Callback set for key: %s", _map_system_settings_key(_ret_type[i].key));
    }

    elm_object_disabled_set(ad->set_cb_bt, EINA_TRUE);
    elm_object_disabled_set(ad->unset_cb_bt, EINA_FALSE);
}

static void _unset_changed_cb(void *data, Evas_Object *obj, void *event_info)
{
    appdata_s *ad = (appdata_s *)data;

    int i;

    for (i = 0; i < SYS_INFO_COUNT; i++) {
        int ret = system_settings_unset_changed_cb(_ret_type[i].key);
        if (ret != SYSTEM_SETTINGS_ERROR_NONE) {
            PRINT_MSG("system_settings_unset_changed_cb error: %d", ret);
            dlog_print(DLOG_ERROR, LOG_TAG, "system_settings_unset_changed_cb error: %d", ret);
        } else
            PRINT_MSG("Callback unset for key: %s", _map_system_settings_key(_ret_type[i].key));
    }

    elm_object_disabled_set(ad->set_cb_bt, EINA_FALSE);
    elm_object_disabled_set(ad->unset_cb_bt, EINA_TRUE);
}

void create_buttons_in_main_window(appdata_s *ad)
{
    Evas_Object *display = _create_new_cd_display(ad, "System Settings", _pop_cb);

    _new_button(ad, display, "Start", _system_settings_cb);
    ad->set_cb_bt = _new_button(ad, display, "Set Changed Callback", _set_changed_cb);
    ad->unset_cb_bt = _new_button(ad, display, "Unset Changed Callback", _unset_changed_cb);

    elm_object_disabled_set(ad->unset_cb_bt, EINA_TRUE);
}
